/*
 * The MIT License (MIT)
 * 
 * Copyright (c) 2015 Camil Staps <info@camilstaps.nl>
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.camilstaps.findfile;

import java.io.File;
import java.io.IOException;

/**
 * Recursively find a file using threads
 * @author Camil Staps s4498062
 */
public class FileFinder implements Runnable {
    
    private final File rootDir;
    private final String search;
    
    /**
     * Create an instance of the FileFinder
     * @param root the name of the directory in which to search
     * @param search the filename to search for
     * @throws IOException if root is not the name of a directory
     */
    public FileFinder(String root, String search) throws IOException {
        rootDir = new File(root);
        if (!rootDir.exists() || !rootDir.isDirectory()) {
            throw new IOException(root + " is not a directory.");
        }
        
        this.search = search;
    }

    @Override
    public void run() {
        find(rootDir, search);
    }
    
    /**
     * Recursively find a file with a name in a directory
     * A new thread is created for every directory encountered
     * @param dir the directory
     * @param name the name
     */
    private void find(File dir, String name) {
        File[] files = dir.listFiles();
        if (files == null) return;
        for (File file : files) {
            if (file.getName().equals(name)) {
                System.out.println(file.getAbsolutePath());
            }
            if (file.isDirectory()) {
                try {
                    Thread t = new Thread(new FileFinder(file.getAbsolutePath(), search));
                    t.start();
                } catch (IOException ex) {}
            }
        }
    }
    
}
